package com.informagen.echo2;

import java.util.EventListener;

import nextapp.echo2.app.TextArea;
import nextapp.echo2.app.text.Document;

import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;

/**
 * A single-line text input field with pre keystroke regular expression testing
 */

public class ActiveTextArea extends TextArea {

    public static final String ACTION_VALIDATION_TRANSACTION = "validation-transition";
    public static final String INPUT_VALID = "inputValid";
    public static final String INPUT_INVALID = "inputInvalid";


    public static final String PROPERTY_MAXIMUM_LENGTH = "maxLength";
    public static final String PROPERTY_MSG_VISIBLE = "messageVisible";

    /**
     * Creates a new <code>RegExTextField</code> with an empty 
     * <code>StringDocument</code> as its model, and default width
     * setting.
    */
    public ActiveTextArea() {}

    
    /**
     * Creates a new <code>RegExTextField</code> with the specified
     * <code>Document</code> model.
     * 
     * @param document the document
    */

    public ActiveTextArea(Document document) {
        super(document);
    }

    
    /**
     * Creates a new <code>RegExTextField</code> with the specified
     * <code>Document</code> model, initial text, and column width.
     * 
     * @param document the document
     * @param text the initial text (may be null)
     * @param columns the number of columns to display
    */
     
    public ActiveTextArea(Document document, String text, int columns, int rows) {
        super(document, text, columns, rows);
    }

    /**
     * @see nextapp.echo2.app.Component#processInput(java.lang.String, java.lang.Object)
     */

    public void processInput(String inputName, Object inputValue) {
        super.processInput(inputName, inputValue);


        if (!hasEventListenerList()) {
            return;
        }

        if(inputName.equals(ACTION_VALIDATION_TRANSACTION))
            processValidationTransition(inputValue);
    }


    private void processValidationTransition(Object inputValue) {
    
        EventListener[] listeners = getEventListenerList().getListeners(ActionListener.class);
 
        ActionEvent e = null;
        
        boolean valid = Boolean.valueOf((String)inputValue).booleanValue();
        
        
        for (int i = 0; i < listeners.length; ++i) {
            // Lazy instanciation
            if (e == null) 
                e = new ActionEvent(this, (valid) ? INPUT_VALID : INPUT_INVALID);
            
            ((ActionListener) listeners[i]).actionPerformed(e);
        }
    
    }


    
    /**
     * Setthe  regular expression which will be checked after each keystroke
     * 
     * @param regex the regular expression
    */
       
    public void setMaxLength(int maxLength) {
        setProperty(PROPERTY_MAXIMUM_LENGTH, new Integer(maxLength));
    }


    /**
     * Get the regular expression for this text field
     * 
     * @returns regex the regular expression
    */
       
    public int getMaxLength() {
        Integer value = (Integer)getProperty(PROPERTY_MAXIMUM_LENGTH);
        return (value != null) ? value.intValue() : -1;
    }


    /**
     * Set the status message to be always visible
     * 
     * @param minValue the minimum value
    */
       
    public void setMessageVisible(boolean isVisible) {
        setProperty(PROPERTY_MSG_VISIBLE, new Boolean(isVisible));
    }


    /**
     * Should the status message always be visible
     * 
     * @returns String the status message, ie Valid values are...
    */
       
    public boolean isMessageVisible() {
        Boolean object = (Boolean)getProperty(PROPERTY_MSG_VISIBLE);
        return (object != null) ? object.booleanValue() : true;
    }


}

