package com.informagen.echo2;

import com.informagen.echo2.ActiveTextField;

import java.lang.NumberFormatException;


/**
 * A single-line text input field with pre keystroke regular expression testing
 */

public class NumericTextField extends ActiveTextField {

    public static final String PROPERTY_MINVALUE = "minimumValue";
    public static final String PROPERTY_MAXVALUE = "maximumValue";



    /**
     * Set the double value
     * 
     * @param value the double value
    */
       
    public void setValue(double value) {
        setText(Double.toString(value));
    }


    /**
     * Get the double value 
     * 
     * @returns double the value
    */
       
    public double getValue() {
        double value;
        try {
            value = Double.parseDouble(getText());
        } catch (NumberFormatException nfe) {
            value = 0;
        }
        
        return value;
    }

    
    /**
     * Set the maximum allowed value
     * 
     * @param maxValue the maximum value
    */
       
    public void setMaxValue(double maxValue) {
        setProperty(PROPERTY_MAXVALUE, new Double(maxValue));
    }


    /**
     * Get the maximum value for this text field
     * 
     * @returns int the maximum value
    */
       
    public double getMaxValue() {
        Double object = (Double)getProperty(PROPERTY_MAXVALUE);
        return (object != null) ? object.doubleValue() : Double.MAX_VALUE;
    }

    
    /**
     * Set the minimum allowed value 
     * 
     * @param minValue the minimum value
    */
       
    public void setMinValue(double minValue) {
        setProperty(PROPERTY_MINVALUE, new Double(minValue));
    }


    /**
     * Get the minimum value for this integer field
     * 
     * @returns int the minimum value
    */
       
    public double getMinValue() {
        Double object = (Double)getProperty(PROPERTY_MINVALUE);
        return (object != null) ? object.doubleValue() : -Double.MAX_VALUE;
    }

}

