package com.informagen.echo2.peers;

import com.informagen.echo2.ActiveTextField;
import com.informagen.echo2.peers.partialupdate.TextUpdate;
import com.informagen.echo2.peers.partialupdate.ForegroundUpdate;
import com.informagen.echo2.peers.partialupdate.BackgroundUpdate;

// NextApp Echo2 Component Framework
import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.FillImage;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Font;
import nextapp.echo2.app.ImageReference;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.TextArea;
import nextapp.echo2.app.TextField;

import nextapp.echo2.app.text.TextComponent;

import nextapp.echo2.app.update.ServerComponentUpdate;

// NextApp Echo2 WebContainer Sub-Framework
import nextapp.echo2.webcontainer.ActionProcessor;
import nextapp.echo2.webcontainer.ContainerInstance;
import nextapp.echo2.webcontainer.FocusSupport;
import nextapp.echo2.webcontainer.PartialUpdateManager;
import nextapp.echo2.webcontainer.PartialUpdateParticipant;
import nextapp.echo2.webcontainer.PropertyUpdateProcessor;
import nextapp.echo2.webcontainer.RenderContext;
import nextapp.echo2.webcontainer.ComponentSynchronizePeer;

import nextapp.echo2.webcontainer.image.ImageRenderSupport;

import nextapp.echo2.webcontainer.partialupdate.BorderUpdate;
import nextapp.echo2.webcontainer.partialupdate.ColorUpdate;
import nextapp.echo2.webcontainer.partialupdate.InsetsUpdate;

// NextApp Echo2 WebRenderer Sub-Framework

import nextapp.echo2.webrender.ClientProperties;
import nextapp.echo2.webrender.ServerMessage;
import nextapp.echo2.webrender.Service;
import nextapp.echo2.webrender.ServiceRegistry;
import nextapp.echo2.webrender.WebRenderServlet;

import nextapp.echo2.webrender.output.CssStyle;

import nextapp.echo2.webrender.servermessage.DomUpdate;
import nextapp.echo2.webrender.servermessage.WindowUpdate;

import nextapp.echo2.webrender.service.StaticBinaryService;

import nextapp.echo2.webrender.service.JavaScriptService;

// NextApp Echo2 WebRenderer CSS renderers
import nextapp.echo2.webcontainer.propertyrender.AlignmentRender;
import nextapp.echo2.webcontainer.propertyrender.FillImageRender;
import nextapp.echo2.webcontainer.propertyrender.BorderRender;
import nextapp.echo2.webcontainer.propertyrender.ColorRender;
import nextapp.echo2.webcontainer.propertyrender.ExtentRender;
import nextapp.echo2.webcontainer.propertyrender.FontRender;
import nextapp.echo2.webcontainer.propertyrender.InsetsRender;

import nextapp.echo2.webrender.util.DomUtil;


// W3C DOM
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


/**
 * Abstract base synchronization peer for the built-in
 * <code>com.informagen.echo2.ActiveTextField</code>
 */

public abstract class ActiveTextFieldPeer implements ActionProcessor, 
                                                     ComponentSynchronizePeer, 
                                                     FocusSupport, 
                                                     ImageRenderSupport, 
                                                     PropertyUpdateProcessor {


    protected static final Alignment DEFAULT_MSG_POSITION = new Alignment(Alignment.TRAILING, Alignment.DEFAULT);
    protected static final Alignment DEFAULT_ICON_POSITION = new Alignment(Alignment.TRAILING, Alignment.DEFAULT);

    private static final String IMAGE_ID_BACKGROUND = "background";

    /**
     * Service to provide supporting images
     */

    /**
     * Service to provide supporting JavaScript library; icons are loaded
     *  and registered in the superclass "ActiveTextFieldPeer"
     */

    static final Service ACTIVE_TEXTFIELD_SERVICE = JavaScriptService.forResource(
                    "Echo.ActiveTextField",
                    "/com/informagen/echo2/resources/js/ActiveTextField.js");


	private static final Service GOOD_ICON_IMAGE = StaticBinaryService.forResource(
	                "Echo.IntegerTextField.goodIcon", 
	                "image/png",
			        "/com/informagen/echo2/resources/images/good.icon.png");

	private static final Service WARNING_ICON_IMAGE = StaticBinaryService.forResource(
	                "Echo.IntegerTextField.warningIcon", 
	                "image/png",
			        "/com/informagen/echo2/resources/images/warning.icon.png");

	private static final Service ERROR_ICON_IMAGE = StaticBinaryService.forResource(
	                "Echo.IntegerTextField.errorIcon", 
	                "image/png",
			        "/com/informagen/echo2/resources/images/error.icon.png");



    static {
		ServiceRegistry registery = WebRenderServlet.getServiceRegistry();
		registery.add(ACTIVE_TEXTFIELD_SERVICE);
		registery.add(GOOD_ICON_IMAGE);
		registery.add(WARNING_ICON_IMAGE);
		registery.add(ERROR_ICON_IMAGE);
    }
 

    ///////////////////////////////////////////////////////////////////////////////////////////

    private PartialUpdateManager partialUpdateManager;

    private String messageProcessor;
    /**
     * Default constructor.
     */

    public ActiveTextFieldPeer(String messageProcessor) {
    
        this.messageProcessor = messageProcessor;
    
        partialUpdateManager = new PartialUpdateManager();

        partialUpdateManager.add(Component.PROPERTY_FOREGROUND, new ForegroundUpdate(messageProcessor));
        partialUpdateManager.add(Component.PROPERTY_BACKGROUND, new BackgroundUpdate(messageProcessor));
        partialUpdateManager.add(TextField.TEXT_CHANGED_PROPERTY, new TextUpdate(messageProcessor));
        
    }


    //====  ImageRenderSupport interface =========================================================

    /**
     * @see nextapp.echo2.webcontainer.image.ImageRenderSupport#getImage(nextapp.echo2.app.Component,
     *      java.lang.String)
     */

    public ImageReference getImage(Component component, String imageId) {
        if (IMAGE_ID_BACKGROUND.equals(imageId)) {
            FillImage backgroundImage;
            if (component.isRenderEnabled()) {
                backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
            } else {
                backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND_IMAGE);
                if (backgroundImage == null) {
                    backgroundImage = (FillImage) component.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
                }
            }
            if (backgroundImage == null) {
                return null;
            } else {
                return backgroundImage.getImage();
            }
        } else {
            return null;
        }
    }


    //====  ActionProcessor interface =========================================================

    /**
     * @see nextapp.echo2.webcontainer.ActionProcessor#processAction(nextapp.echo2.webcontainer.ContainerInstance, 
     *      nextapp.echo2.app.Component, org.w3c.dom.Element)
     */
    public void processAction(ContainerInstance ci, Component component, Element actionElement) {
    
        String name = actionElement.getAttribute("name");
        String value = actionElement.getAttribute("value");
        
        ci.getUpdateManager().getClientUpdateManager().setComponentAction(component, name, value);
        
        //ci.getUpdateManager().getClientUpdateManager().setComponentAction(component, TextField.INPUT_ACTION, null);
    }



    //====  PropertyUpdateProcessor interface =================================================

    /**
     * @see nextapp.echo2.webcontainer.PropertyUpdateProcessor#processPropertyUpdate(
     *      nextapp.echo2.webcontainer.ContainerInstance,
     *      nextapp.echo2.app.Component, org.w3c.dom.Element)
     */
    public void processPropertyUpdate(ContainerInstance ci, Component component, Element propertyElement) {
    
        String propertyName = propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_NAME);

        if (TextField.TEXT_CHANGED_PROPERTY.equals(propertyName)) {

            String propertyValue = DomUtil.getElementText(propertyElement);
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.TEXT_CHANGED_PROPERTY, propertyValue);

        } else if (TextField.PROPERTY_HORIZONTAL_SCROLL.equals(propertyName)) {

            Extent propertyValue = new Extent(Integer.parseInt(
                    propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_VALUE)));
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.PROPERTY_HORIZONTAL_SCROLL, propertyValue);

        } else if (TextField.PROPERTY_VERTICAL_SCROLL.equals(propertyName)) {

            Extent propertyValue = new Extent(Integer.parseInt(
                    propertyElement.getAttribute(PropertyUpdateProcessor.PROPERTY_VALUE)));
            ci.getUpdateManager().getClientUpdateManager().setComponentProperty(component, 
                    TextField.PROPERTY_VERTICAL_SCROLL, propertyValue);
        }
    }

    //====  FocusSupport interface ============================================================

    /**
     * @see nextapp.echo2.webcontainer.FocusSupport#renderSetFocus(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.Component)
     */
    public void renderSetFocus(RenderContext rc, Component component) {
        WindowUpdate.renderSetFocus(rc.getServerMessage(), ContainerInstance.getElementId(component));
    }



    //==== ComponentSychronizePeer interface ==================================================
    //  - getContainerId
    //  - renderAdd
    //  - renderUpdate
    //  - renderDispose
    
    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#getContainerId(nextapp.echo2.app.Component)
     */
    public String getContainerId(Component child) {
        throw new UnsupportedOperationException("Component does not support children.");
    }


    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderAdd(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String,
     *      nextapp.echo2.app.Component)
     */
    public void renderAdd(RenderContext rc, ServerComponentUpdate update, String targetId, Component component) {

        Element domAddElement = DomUpdate.renderElementAdd(rc.getServerMessage());
        DocumentFragment htmlFragment = rc.getServerMessage().getDocument().createDocumentFragment();

        renderHtml(rc, update, htmlFragment, component);

        DomUpdate.renderElementAddContent(rc.getServerMessage(), domAddElement, targetId, htmlFragment);
    }



    public void renderDispose(RenderContext rc, ServerComponentUpdate update, Component component) {

        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(ACTIVE_TEXTFIELD_SERVICE.getId());

        renderDisposeDirective(rc, component);
    }

    protected void renderDisposeDirective(RenderContext rc, Component component) {
    
        String elementId = ContainerInstance.getElementId(component);
        
        ServerMessage serverMessage = rc.getServerMessage();
        Element itemizedUpdateElement = serverMessage.getItemizedDirective(ServerMessage.GROUP_ID_PREREMOVE,
                messageProcessor, "dispose", new String[0], new String[0]);
                
        Element itemElement = serverMessage.getDocument().createElement("item");
        
        itemElement.setAttribute("eid", elementId);
        itemizedUpdateElement.appendChild(itemElement);
    }


    /**
     * Renders a directive to the outgoing <code>ServerMessage</code> to
     * initialize the state of a text component, performing tasks such as
     * registering event listeners on the client.
     * 
     * @param rc the relevant <code>RenderContext</code>
     * @param integerTextField the <code>LiveTextField<code>
     */

    protected Element renderInitDirective(RenderContext rc, ActiveTextField activeTextField) {
    
        String elementId = ContainerInstance.getElementId(activeTextField);
        ServerMessage serverMessage = rc.getServerMessage();

        Element itemizedUpdateElement = serverMessage.getItemizedDirective(ServerMessage.GROUP_ID_POSTUPDATE,
                messageProcessor, "init", new String[0], new String[0]);
 
  
        Element itemElement = serverMessage.getDocument().createElement("item");
        itemElement.setAttribute("eid", elementId);

        Extent horizontalScroll = (Extent) activeTextField.getRenderProperty(ActiveTextField.PROPERTY_HORIZONTAL_SCROLL);
        Extent verticalScroll = (Extent) activeTextField.getRenderProperty(ActiveTextField.PROPERTY_VERTICAL_SCROLL);
        
        if (horizontalScroll != null && horizontalScroll.getValue() != 0) {
            itemElement.setAttribute("horizontal-scroll", ExtentRender.renderCssAttributePixelValue(horizontalScroll, "0"));
        }
        if (verticalScroll != null && verticalScroll.getValue() != 0) {
            itemElement.setAttribute("vertical-scroll", ExtentRender.renderCssAttributePixelValue(verticalScroll, "0"));
        }
        
        if (!activeTextField.isRenderEnabled()) {
            itemElement.setAttribute("enabled", "false");
        }

        String message = activeTextField.getMessage();
        if (message != null) 
            itemElement.setAttribute("status-msg", message);

        String text = activeTextField.getText();
        if (text != null) 
            itemElement.setAttribute("text", text);

        itemElement.setAttribute("msg-visible", activeTextField.isMessageVisible() ? "true" : "false");
        itemElement.setAttribute("icon-visible", activeTextField.isIconVisible() ? "true" : "false");
        
        itemElement.setAttribute("msg-always-visible", activeTextField.isMessageAlwaysVisible() ? "true" : "false");
        itemElement.setAttribute("icon-always-visible", activeTextField.isIconAlwaysVisible() ? "true" : "false");


        itemElement.setAttribute("foreground", ColorRender.renderCssAttributeValue((Color)activeTextField.getRenderProperty(Component.PROPERTY_FOREGROUND)));
        itemElement.setAttribute("background", ColorRender.renderCssAttributeValue((Color)activeTextField.getRenderProperty(Component.PROPERTY_BACKGROUND)));


        if (activeTextField.getErrorForeground() != null && !activeTextField.getErrorForeground().equals("")) 
            itemElement.setAttribute("error-foreground", ColorRender.renderCssAttributeValue(activeTextField.getErrorForeground()));

        if (activeTextField.getErrorBackground() != null && !activeTextField.getErrorBackground().equals("")) 
            itemElement.setAttribute("error-background", ColorRender.renderCssAttributeValue(activeTextField.getErrorBackground()));

        if (activeTextField.hasActionListeners())
            itemElement.setAttribute("server-notify", "true");

        itemizedUpdateElement.appendChild(itemElement);
        
        return itemElement;
    }


    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderUpdate(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate, java.lang.String)
     */
    public boolean renderUpdate(RenderContext rc, ServerComponentUpdate update, String targetId) {
        boolean fullReplace = false;
        if (update.hasUpdatedProperties()) {
            if (!partialUpdateManager.canProcess(rc, update)) {
                fullReplace = true;
            }
        }

        if (fullReplace) {
            // Perform full update.
            DomUpdate.renderElementRemove(rc.getServerMessage(), ContainerInstance.getElementId(update.getParent()));
            renderAdd(rc, update, targetId, update.getParent());
        } else {
            partialUpdateManager.process(rc, update);
        }

        return false;
    }



    /*====  DomUpdateSupport interface ========================================================

    
    /**
     * @see nextapp.echo2.webcontainer.DomUpdateSupport#renderHtml(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate, org.w3c.dom.Node, nextapp.echo2.app.Component)
     */
    public void renderHtml(RenderContext rc, ServerComponentUpdate addUpdate, Node parentNode, Component component) {
        
        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(ACTIVE_TEXTFIELD_SERVICE.getId());

    
        ActiveTextField textField = (ActiveTextField) component;
        String elementId = ContainerInstance.getElementId(textField);

        // Create the outer <div> with element id
        
 		Element divOuter = parentNode.getOwnerDocument().createElement("div");
		divOuter.setAttribute("id", elementId);

        // Create the component parts
        Element inputElement = parentNode.getOwnerDocument().createElement("input");
		inputElement.setAttribute("id", elementId + ".input");
        inputElement.setAttribute("type", "text");


        String text = textField.getText();
        if (text != null) 
            inputElement.setAttribute("value", text);

        
        if (textField.isFocusTraversalParticipant()) {
            inputElement.setAttribute("tabindex", Integer.toString(textField.getFocusTraversalIndex()));
        } else {
            inputElement.setAttribute("tabindex", "-1");
        }
        
        String toolTipText = (String) textField.getRenderProperty(TextField.PROPERTY_TOOL_TIP_TEXT);
        if (toolTipText != null) {
            inputElement.setAttribute("title", toolTipText);
        }
        
        Integer maximumLength = (Integer) textField.getRenderProperty(TextField.PROPERTY_MAXIMUM_LENGTH);
        if (maximumLength != null) {
            inputElement.setAttribute("maxlength", maximumLength.toString());
        }

        CssStyle cssStyle = createBaseCssStyle(rc, textField);
        
        if (cssStyle.hasAttributes())
            inputElement.setAttribute("style", cssStyle.renderInline());
        

        // The 'OK' icon
        String srcURI = rc.getContainerInstance().getServiceUri(GOOD_ICON_IMAGE);
		Element goodIconElement = parentNode.getOwnerDocument().createElement("img");
		goodIconElement.setAttribute("id", elementId + ".good");
		goodIconElement.setAttribute("src", srcURI);
		goodIconElement.setAttribute("width", "13");
		goodIconElement.setAttribute("height", "13");
		goodIconElement.setAttribute("alt", "OK");

		cssStyle = new CssStyle();
		cssStyle.setAttribute("vertical-align", "middle");
		cssStyle.setAttribute("padding-left", "3px");
		cssStyle.setAttribute("padding-right", "3px");
		cssStyle.setAttribute("display", (textField.isIconAlwaysVisible() ? "" : "none"));
		goodIconElement.setAttribute("style", cssStyle.renderInline());


        // The 'error' icon
        srcURI = rc.getContainerInstance().getServiceUri(ERROR_ICON_IMAGE);
		Element errorIconElement = parentNode.getOwnerDocument().createElement("img");
		errorIconElement.setAttribute("id", elementId + ".error");
		errorIconElement.setAttribute("src", srcURI);
		errorIconElement.setAttribute("width", "13");
		errorIconElement.setAttribute("height", "13");
		errorIconElement.setAttribute("alt", "Error");
		
		cssStyle = new CssStyle();
		cssStyle.setAttribute("vertical-align", "middle");
		cssStyle.setAttribute("padding-left", "3px");
		cssStyle.setAttribute("padding-right", "3px");
		cssStyle.setAttribute("display", "none");
		errorIconElement.setAttribute("style", cssStyle.renderInline());


        
        Element messageSpan = parentNode.getOwnerDocument().createElement("span");
		messageSpan.setAttribute("id", elementId + ".msg");
		
		cssStyle = new CssStyle();
		cssStyle.setAttribute("vertical-align", "middle");
		cssStyle.setAttribute("padding-left", "3px");
		cssStyle.setAttribute("padding-right", "3px");
		cssStyle.setAttribute("font-family", "sans-serif");
		cssStyle.setAttribute("font-size", "x-small");
		cssStyle.setAttribute("display", (textField.isMessageAlwaysVisible() ? "" : "none"));
		messageSpan.setAttribute("style", cssStyle.renderInline());


        // Assemble the component parts starting with the <input>

		parentNode.appendChild(divOuter);

 		Element divInner = parentNode.getOwnerDocument().createElement("div");
        divOuter.appendChild(divInner);
        divInner.appendChild(inputElement);


        Alignment iconPosition = (Alignment) textField.getRenderProperty(ActiveTextField.PROPERTY_ICON_POSITION, 
                                                                         DEFAULT_ICON_POSITION);
        int iconOrientation = convertAlignmentToPosition(iconPosition, textField);

        // Add the icons
        switch(iconOrientation) {
            
            case Alignment.TOP:
                divOuter.insertBefore(goodIconElement, divInner);
                divOuter.insertBefore(errorIconElement, divInner);
                break;
                
            case Alignment.RIGHT:
                divInner.appendChild(goodIconElement);
                divInner.appendChild(errorIconElement);
                break;
                
            case Alignment.BOTTOM:
                divOuter.appendChild(goodIconElement);
                divOuter.appendChild(errorIconElement);
                break;
                
            case Alignment.LEFT:
                divInner.insertBefore(goodIconElement, inputElement);
                divInner.insertBefore(errorIconElement, inputElement);
                break;
                
        }

        // Add the status message
        Alignment textPosition = (Alignment) textField.getRenderProperty(ActiveTextField.PROPERTY_MSG_POSITION, 
                                                                             DEFAULT_MSG_POSITION);
        int textOrientation = convertAlignmentToPosition(textPosition, textField);

        switch(textOrientation) {
            
            case Alignment.TOP:
                divOuter.insertBefore(messageSpan, divInner);
                break;

            case Alignment.RIGHT:
                divInner.appendChild(messageSpan);
                break;

            case Alignment.BOTTOM:
                divOuter.appendChild(messageSpan);
                break;
                
            case Alignment.LEFT:
                divInner.insertBefore(messageSpan, inputElement);
                break;
        }
         
    }


    /**
     * Creates a base <code>CssStyle</code> for properties common to text
     * components.
     * 
     * @param rc the relevant <code>RenderContext</code>
     * @param integerTextField the text component
     * @return the style
     */

    protected CssStyle createBaseCssStyle(RenderContext rc, TextField textField) {
        CssStyle cssStyle = new CssStyle();

        boolean renderEnabled = textField.isRenderEnabled();

        Border border;
        Color foreground, background;
        Font font;
        FillImage backgroundImage;
        if (!renderEnabled) {
            // Retrieve disabled style information.
            background = (Color) textField.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND);
            backgroundImage = (FillImage) textField.getRenderProperty(TextField.PROPERTY_DISABLED_BACKGROUND_IMAGE);
            border = (Border) textField.getRenderProperty(TextField.PROPERTY_DISABLED_BORDER);
            font = (Font) textField.getRenderProperty(TextField.PROPERTY_DISABLED_FONT);
            foreground = (Color) textField.getRenderProperty(TextField.PROPERTY_DISABLED_FOREGROUND);

            // Fallback to normal styles.
            if (background == null) {
                background = (Color) textField.getRenderProperty(Component.PROPERTY_BACKGROUND);
                if (backgroundImage == null) {
                    // Special case:
                    // Disabled background without disabled background image will render disabled background instead of
                    // normal background image.
                    backgroundImage = (FillImage) textField.getRenderProperty(TextField.PROPERTY_BACKGROUND_IMAGE);
                }
            }
            if (border == null) {
                border = (Border) textField.getRenderProperty(TextField.PROPERTY_BORDER);
            }
            if (font == null) {
                font = (Font) textField.getRenderProperty(TextField.PROPERTY_FONT);
            }
            if (foreground == null) {
                foreground = (Color) textField.getRenderProperty(Component.PROPERTY_FOREGROUND);
            }
        } else {
            border = (Border) textField.getRenderProperty(TextField.PROPERTY_BORDER);
            foreground = (Color) textField.getRenderProperty(Component.PROPERTY_FOREGROUND);
            background = (Color) textField.getRenderProperty(Component.PROPERTY_BACKGROUND);
            font = (Font) textField.getRenderProperty(TextField.PROPERTY_FONT);
        }
        
        Alignment alignment = (Alignment) textField.getRenderProperty(TextField.PROPERTY_ALIGNMENT);
        
        if (alignment != null) {
            int horizontalAlignment = AlignmentRender.getRenderedHorizontal(alignment, textField);
            switch (horizontalAlignment) {
            case Alignment.LEFT:
                cssStyle.setAttribute("text-align", "left");
                break;
            case Alignment.CENTER:
                cssStyle.setAttribute("text-align", "center");
                break;
            case Alignment.RIGHT:
                cssStyle.setAttribute("text-align", "right");
                break;
            }
        }
        
        BorderRender.renderToStyle(cssStyle, border);
        ColorRender.renderToStyle(cssStyle, foreground, background);
        FontRender.renderToStyle(cssStyle, font);
        
        
        InsetsRender.renderToStyle(cssStyle, "padding", (Insets) textField.getRenderProperty(TextField.PROPERTY_INSETS));
        
        Extent width = (Extent) textField.getRenderProperty(TextField.PROPERTY_WIDTH);
        Extent height = (Extent) textField.getRenderProperty(TextField.PROPERTY_HEIGHT);

        if (width != null) {
            cssStyle.setAttribute("width", ExtentRender.renderCssAttributeValue(width));
        }

        if (height != null) {
            cssStyle.setAttribute("height", ExtentRender.renderCssAttributeValue(height));
        }
        return cssStyle;
    }


    static int convertAlignmentToPosition(Alignment position, Component component) {

        if (position.getVertical() == Alignment.DEFAULT) {
        
            switch (position.getHorizontal()) {
                case Alignment.LEFT:
                    return Alignment.LEFT;
                case Alignment.RIGHT:
                    return Alignment.RIGHT;
                case Alignment.LEADING:
                    return component.getRenderLayoutDirection().isLeftToRight() 
                        ? Alignment.LEFT : Alignment.RIGHT;
                case Alignment.TRAILING:
                    return component.getRenderLayoutDirection().isLeftToRight() 
                        ? Alignment.RIGHT : Alignment.LEFT;
                default:
                    // Invalid, return value for TRAILING (default).
                    return Alignment.RIGHT;
            }
        } else {
            if (position.getVertical() == Alignment.TOP) {
                return Alignment.TOP;
            } else {
                return Alignment.BOTTOM;
            }
        }
    }


}
