package com.informagen.echo2.peers;


// Our Echo2 component
import com.informagen.echo2.NumericTextField;

// NextApp Echo2 Component Framework
import nextapp.echo2.app.Component;

import nextapp.echo2.app.update.ServerComponentUpdate;


// NextApp Echo2 WebContainer Framework
import nextapp.echo2.webcontainer.RenderContext;
import nextapp.echo2.webcontainer.DomUpdateSupport;


// NextApp Echo2 WebRenderer Sub-Framework
import nextapp.echo2.webrender.ServerMessage;
import nextapp.echo2.webrender.Service;
import nextapp.echo2.webrender.ServiceRegistry;
import nextapp.echo2.webrender.WebRenderServlet;

import nextapp.echo2.webrender.service.JavaScriptService;


// W3C DOM
import org.w3c.dom.Node;
import org.w3c.dom.Element;


/**
 * Synchronization peer for the 
 * <code>com.informagen.echo2.IntegerTextField</code> component
 */


public class NumericTextFieldPeer extends ActiveTextFieldPeer
                                  implements DomUpdateSupport {

    /**
     * Service to provide supporting JavaScript library; icons are loaded
     *  and registered in the superclass "ActiveTextFieldPeer"
     */

    static final Service NUMERIC_TEXTFIELD_SERVICE = JavaScriptService.forResource(
                    "Echo.NumericTextField",
                    "/com/informagen/echo2/resources/js/NumericTextField.js");


    static {
		ServiceRegistry registery = WebRenderServlet.getServiceRegistry();
		registery.add(NUMERIC_TEXTFIELD_SERVICE);
    }
 

    ///////////////////////////////////////////////////////////////////////////////////////////


    /**
     * Default constructor.
     */

    public NumericTextFieldPeer() {
        super("NumericTextField.MessageProcessor");
    }


    /**
     * @see nextapp.echo2.webcontainer.ComponentSynchronizePeer#renderDispose(nextapp.echo2.webcontainer.RenderContext,
     *      nextapp.echo2.app.update.ServerComponentUpdate,
     *      nextapp.echo2.app.Component)
     */
    public void renderDispose(RenderContext rc, ServerComponentUpdate update, Component component) {

        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(NUMERIC_TEXTFIELD_SERVICE.getId());

        super.renderDispose(rc, update, component);
    }


    
    /**
     * @see nextapp.echo2.webcontainer.DomUpdateSupport#renderHtml(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate, org.w3c.dom.Node, nextapp.echo2.app.Component)
     */

    public void renderHtml(RenderContext rc, ServerComponentUpdate addUpdate, Node parentNode, Component component) {
    
        ServerMessage serverMessage = rc.getServerMessage();
        serverMessage.addLibrary(NUMERIC_TEXTFIELD_SERVICE.getId());

        NumericTextField numericTextField = (NumericTextField)component;
        
        super.renderHtml(rc, addUpdate, parentNode, numericTextField);


        // Now build a generic 'init' message directive, then add this subclasses's
        //   specific attributes
        
        Element itemElement = renderInitDirective(rc, numericTextField);

        itemElement.setAttribute("minimum-value", Double.toString(numericTextField.getMinValue()));
        itemElement.setAttribute("maximum-value", Double.toString(numericTextField.getMaxValue()));

    }


}
