
package com.informagen.echo2.peers.partialupdate;


// NextApp Echo2 Framework
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.update.ServerComponentUpdate;


// NextApp Echo2 WebContainer Sub-Framework
import nextapp.echo2.webcontainer.ContainerInstance;
import nextapp.echo2.webcontainer.PartialUpdateParticipant;
import nextapp.echo2.webcontainer.RenderContext;

// NextApp Echo2 WebRenderer Sub-Framework
import nextapp.echo2.webrender.ServerMessage;
import nextapp.echo2.webcontainer.propertyrender.ColorRender;

// W3C DOM
import org.w3c.dom.Element;

 
/**
 * A <code>PartialUpdateParticipant</code> to update the text any
 * subclass of 'TextComponent'
 */

public class BackgroundUpdate implements PartialUpdateParticipant {
 
    private String messageProcessor;

    public BackgroundUpdate(String messageProcessor) {
        this.messageProcessor = messageProcessor;
    }
    
    
    /**
     * @see nextapp.echo2.webcontainer.PartialUpdateParticipant#canRenderProperty(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate)
     */
    public boolean canRenderProperty(RenderContext rc, ServerComponentUpdate update) {
        return true;
    }

    /**
     * @see nextapp.echo2.webcontainer.PartialUpdateParticipant#renderProperty(
     *      nextapp.echo2.webcontainer.RenderContext, nextapp.echo2.app.update.ServerComponentUpdate)
     */
    public void renderProperty(RenderContext rc, ServerComponentUpdate update) {
    
        Component component = (Component) update.getParent();
        
        String elementId = ContainerInstance.getElementId(component);
        ServerMessage serverMessage = rc.getServerMessage();
        
        Element itemizedUpdateElement = serverMessage.getItemizedDirective(
                ServerMessage.GROUP_ID_POSTUPDATE,
                messageProcessor, 
                "set-background", 
                new String[0], 
                new String[0]);
                
        Element itemElement = serverMessage.getDocument().createElement("item");
        itemElement.setAttribute("eid", elementId);
        
        Color color = (Color)component.getRenderProperty(Component.PROPERTY_BACKGROUND);

        if(color == null)
            color = Color.WHITE;
        
        itemElement.setAttribute("background", ColorRender.renderCssAttributeValue(color));
        
        itemizedUpdateElement.appendChild(itemElement);
        
    }
}

