
package com.informagen.echo2.peers.partialupdate;


// NextApp Echo2 Framework
import nextapp.echo2.app.text.TextComponent;
import nextapp.echo2.app.update.ServerComponentUpdate;


// NextApp Echo2 WebContainer Sub-Framework
import nextapp.echo2.webcontainer.ContainerInstance;
import nextapp.echo2.webcontainer.PartialUpdateParticipant;
import nextapp.echo2.webcontainer.RenderContext;

// NextApp Echo2 WebRenderer Sub-Framework
import nextapp.echo2.webrender.ServerMessage;

// W3C DOM
import org.w3c.dom.Element;

 
/**
 * A <code>PartialUpdateParticipant</code> to update the text any
 * subclass of 'TextComponent'
 */

public class TextUpdate implements PartialUpdateParticipant {

    private String messageProcessor;


    /**
     * Will update the only text of a TextComponent with creating a new cleint-side
     *  component.  This is know as a partial update.
     * 
     * @param messageProcessor the name of the client-side JavaScript message processor
     */


    public TextUpdate(String messageProcessor) {
        this.messageProcessor = messageProcessor;
    }


    /**
     * @see nextapp.echo2.webcontainer.PartialUpdateParticipant#canRenderProperty(nextapp.echo2.webcontainer.RenderContext, 
     *      nextapp.echo2.app.update.ServerComponentUpdate)
     */
    public boolean canRenderProperty(RenderContext rc, ServerComponentUpdate update) {
        return true;
    }

    /**
     * @see nextapp.echo2.webcontainer.PartialUpdateParticipant#renderProperty(
     *      nextapp.echo2.webcontainer.RenderContext, nextapp.echo2.app.update.ServerComponentUpdate)
     */

    public void renderProperty(RenderContext rc, ServerComponentUpdate update) {
    
        TextComponent textComponent = (TextComponent) update.getParent();
        
        String elementId = ContainerInstance.getElementId(textComponent);
        ServerMessage serverMessage = rc.getServerMessage();
        
        Element itemizedUpdateElement = serverMessage.getItemizedDirective(
                ServerMessage.GROUP_ID_POSTUPDATE,
                messageProcessor, 
                "set-text", 
                new String[0], 
                new String[0]);
                
        Element itemElement = serverMessage.getDocument().createElement("item");
        itemElement.setAttribute("eid", elementId);
        itemElement.setAttribute("text", textComponent.getText());
        
        itemizedUpdateElement.appendChild(itemElement);
        
    }
}

