package com.informagen.starter.echo;

import com.informagen.starter.echo.EchoApplication;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import org.apache.log4j.Logger;

public class ServletListener implements HttpSessionListener {
	
	// Application instance constant
	public static final String APPLICATION_INSTANCE = "application-instance";

	
	// The logger
	private static Logger logger = Logger.getLogger(ServletListener.class.getName());
	

	/**
	 * Session is created, nothing can really be logged except that
	 *  a session was create, we don't yet have any login information
	 */
	public void sessionCreated(HttpSessionEvent event) {
        logger.debug("An Echo2 session was created");
    }
	

	/**
	 * Session is shutdown i.e. 'destroyed'
	 *
	 *  This method can be invoked two different ways:
	 *     1) Normal application driven termination
	 *     2) Servlet timeout, user walked off or quit browser
	 *
	 */

	public void sessionDestroyed(HttpSessionEvent event) {
		
		//Get the session and application instance
		
 		HttpSession session = event.getSession();
 		EchoApplication application = (EchoApplication) session.getAttribute(APPLICATION_INSTANCE);
 		
 		
 		// Test for null in case the application was never registered or if we are
 		//   running with Jetty
 		
 		// We release resources here in case the servlet timed-out.  Normally,
 		//   resources are released by normal application termination
 		
        if(application != null) {
 		    application.releaseResources();
            logger.error("An Echo2 session was shutdown after a servlet timeout");
 		    session.removeAttribute(APPLICATION_INSTANCE);
        } else
            logger.debug("An Echo2 session was shutdown normally");
	}

}
