/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2005 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo2.testapp.interactive.testscreen;

import nextapp.echo2.app.FillImage;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.Column;
import nextapp.echo2.app.Label;
import nextapp.echo2.app.SplitPane;

import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;
import nextapp.echo2.app.event.DocumentEvent;
import nextapp.echo2.app.event.DocumentListener;

import nextapp.echo2.app.layout.SplitPaneLayoutData;
import nextapp.echo2.testapp.interactive.ButtonColumn;

import nextapp.echo2.testapp.interactive.InteractiveApp;
import nextapp.echo2.testapp.interactive.StyleUtil;
import nextapp.echo2.testapp.interactive.Styles;


// This test component
import com.informagen.echo2.ActiveTextArea;

public class ActiveTextAreaTest extends SplitPane {
    
    /**
     * Writes <code>ActionEvent</code>s to console.
     */
    private ActionListener actionListener = new ActionListener() {

        /**
         * @see nextapp.echo2.app.event.ActionListener#actionPerformed(nextapp.echo2.app.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            String action = e.getActionCommand();                
            ((InteractiveApp) getApplicationInstance()).consoleWrite(action);
        }
    };
    
    /**
     * Writes <code>ActionEvent</code>s to console.
     */
    private DocumentListener documentListener = new DocumentListener() {
        
        /**
         * @see nextapp.echo2.app.event.DocumentListener#documentUpdate(nextapp.echo2.app.event.DocumentEvent)
         */
        public void documentUpdate(DocumentEvent e) {
            ;//((InteractiveApp) getApplicationInstance()).consoleWrite(e.toString());
        }
    };

    final ActiveTextArea activeTextArea = new ActiveTextArea();

    
    public ActiveTextAreaTest() {
        super(SplitPane.ORIENTATION_HORIZONTAL, new Extent(250, Extent.PX));
        
        setStyleName("DefaultResizable");

        SplitPaneLayoutData splitPaneLayoutData;
        
        // Control Groups in the splitpane's left side
        Column controlGroupsColumn = new Column();
        controlGroupsColumn.setStyleName("TestControlsColumn");
        add(controlGroupsColumn);

        // Build the testing panel in the splitpane's right side
        add(buildTestingPanel());


        // Build the test control buttons as small groups (controlsColumn) 
        //   placed within a larger group (controlGroupsColumn) 
        ButtonColumn controlsColumn;

                
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("ActiveTextArea Properties"));

        controlsColumn.addButton("MaxLength = 20", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                 activeTextArea.setMaxLength(20);
            }
        });

        controlsColumn.addButton("MaxLength = 60", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                 activeTextArea.setMaxLength(60);
            }
        });

        controlsColumn.addButton("MaxLength = 100", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                 activeTextArea.setMaxLength(100);
            }
        });

        controlsColumn.addButton("MaxLength = -1 (no limit)", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                 activeTextArea.setMaxLength(-1);
            }
        });

        controlsColumn.addButton("Toggle Status Visible", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextArea.isMessageVisible();
                 activeTextArea.setMessageVisible(enabled);
            }
        });



        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Set Text"));

        controlsColumn.addButton("Hello, World!", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setText("Hello, World!");
            }
        });

        controlsColumn.addButton("Empty String", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setText("");
            }
        });
        
        controlsColumn.addButton("null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setText(null);
            }
        });

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Width"));

        controlsColumn.addButton("null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setWidth(null);
            }
        });

        controlsColumn.addButton("35 em", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setWidth(new Extent(35, Extent.EM));
            }
        });

        controlsColumn.addButton("100%", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setWidth(new Extent(100, Extent.PERCENT));
            }
        });



        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Listeners"));

        controlsColumn.addButton("Get Text", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String text = activeTextArea.getText();
                ((InteractiveApp) getApplicationInstance()).consoleWrite(text);
            }
        });
        

        controlsColumn.addButton("Add ActionListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.addActionListener(actionListener);
            }
        });
        controlsColumn.addButton("Remove ActionListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.removeActionListener(actionListener);
            }
        });
        
/*
        controlsColumn.addButton("Add DocumentListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.getDocument().addDocumentListener(documentListener);
                //passwordField.getDocument().addDocumentListener(documentListener);
                //textArea.getDocument().addDocumentListener(documentListener);
            }
        });
        controlsColumn.addButton("Remove DocumentListener", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.getDocument().removeDocumentListener(documentListener);
                //passwordField.getDocument().removeDocumentListener(documentListener);
                //textArea.getDocument().removeDocumentListener(documentListener);
            }
        });
*/





/*        
        controlsColumn.addButton("Change Border (All Attributes)", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.randomBorder();
                activeTextArea.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Color", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = activeTextArea.getBorder();
                if (border == null) {
                    return;
                }
                border = new Border(border.getSize(), StyleUtil.randomColor(), border.getStyle());
                activeTextArea.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Size", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderSize(activeTextArea.getBorder());
                if (border == null) {
                    return;
                }
                activeTextArea.setBorder(border);
            }
        });

        controlsColumn.addButton("Change Border Style", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderStyle(activeTextArea.getBorder());
                if (border == null) {
                    return;
                }
                activeTextArea.setBorder(border);
            }
        });
*/


/*
        controlsColumn.addButton("Toggle Background Image", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FillImage backgroundImage = activeTextArea.getBackgroundImage();
                if (backgroundImage == null) {
                    activeTextArea.setBackgroundImage(Styles.BG_SHADOW_LIGHT_BLUE);
                } else {
                    activeTextArea.setBackgroundImage(null);
                }
            }
        });
*/

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Foreground & Background"));

        controlsColumn.addButton("Random Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextArea.setForeground(color);
            }
        });
        controlsColumn.addButton("Reset Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setForeground(null);
            }
        });

        controlsColumn.addButton("Random Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextArea.setBackground(color);
            }
        });

        controlsColumn.addButton("Reset Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setBackground(null);
            }
        });

/*
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Border"));
        controlsColumn.addButton("Change Disabled Border (All Attributes)", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.randomBorder();
                activeTextArea.setDisabledBorder(border);
            }
        });

        controlsColumn.addButton("Change Disabled Border Color", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = activeTextArea.getDisabledBorder();
                if (border == null) {
                    return;
                }
                border = new Border(border.getSize(), StyleUtil.randomColor(), border.getStyle());
                activeTextArea.setDisabledBorder(border);
            }
        });
        
        controlsColumn.addButton("Change Disabled Border Size", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderSize(activeTextArea.getDisabledBorder());
                if (border == null) {
                    return;
                }
                activeTextArea.setDisabledBorder(border);
            }
        });
        
        controlsColumn.addButton("Change Disabled Border Style", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = StyleUtil.nextBorderStyle(activeTextArea.getDisabledBorder());
                if (border == null) {
                    return;
                }
                activeTextArea.setDisabledBorder(border);
            }
        });
*/

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Enabled/Disabled"));

        controlsColumn.addButton("Toggle Enabled", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !activeTextArea.isEnabled();
                activeTextArea.setEnabled(enabled);
            }
        });
/*
        controlsColumn.addButton("Toggle Disabled Background Image", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FillImage backgroundImage = activeTextArea.getDisabledBackgroundImage();
                if (backgroundImage == null) {
                    activeTextArea.setDisabledBackgroundImage(Styles.BG_SHADOW_LIGHT_BLUE);
                } else {
                    activeTextArea.setDisabledBackgroundImage(null);
                }
            }
        });
*/


/*
        controlsColumn.addButton("Set Disabled Foreground", new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextArea.setDisabledForeground(color);
            }
        });

        controlsColumn.addButton("Clear Disabled Foreground", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setDisabledForeground(null);
            }
        });
        
        controlsColumn.addButton("Set Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color color = StyleUtil.randomColor();
                activeTextArea.setDisabledBackground(color);
            }
        });

        controlsColumn.addButton("Clear Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setDisabledBackground(null);
            }
        });
*/

/*
        controlsColumn.addButton("Insets -> null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setInsets(null);
            }
        });

        controlsColumn.addButton("Insets -> 0px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setInsets(new Insets(0));
            }
        });

        controlsColumn.addButton("Insets -> 5px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setInsets(new Insets(5));
            }
        });

        controlsColumn.addButton("Insets -> 10/20/30/40px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                activeTextArea.setInsets(new Insets(10, 20, 30, 40));
            }
        });
*/

        controlsColumn.addButton("Toggle ToolTip Enabled", new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                if (activeTextArea.getToolTipText() == null) {
                    activeTextArea.setToolTipText("Active text area");
                } else {
                    activeTextArea.setToolTipText(null);
                }
            }
        });


    }

    private Component buildTestingPanel() {
    
        Column testColumn = new Column();
        testColumn.setStyleName("TestColumn");
        
        activeTextArea.setWidth(new Extent(40, Extent.EM));
        activeTextArea.setHeight(new Extent(20, Extent.EM));

        testColumn.add(activeTextArea);
        
        return testColumn;
    }


}
