package nextapp.echo2.testapp.interactive.testscreen;

import java.util.Locale;

import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Font;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.Label;
import nextapp.echo2.app.Column;
import nextapp.echo2.app.LayoutDirection;
import nextapp.echo2.app.SplitPane;
import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;
import nextapp.echo2.app.layout.SplitPaneLayoutData;
import nextapp.echo2.testapp.interactive.ButtonColumn;
import nextapp.echo2.testapp.interactive.StyleUtil;
import nextapp.echo2.testapp.interactive.Styles;

// This test component
import com.informagen.echo2.HorizontalRule;


/**
 * An interactive test for <code>HorizontalRule</code>s.
 */
 
public class HorizontalRuleTest extends SplitPane {
        
    final HorizontalRule horizontalRule = new HorizontalRule();
    
    public HorizontalRuleTest() {
        super(SplitPane.ORIENTATION_HORIZONTAL, new Extent(250, Extent.PX));
        
        setStyleName("DefaultResizable");
        
        // Control Groups in the splitpane's left side
        Column controlGroupsColumn = new Column();
        controlGroupsColumn.setStyleName("TestControlsColumn");
        add(controlGroupsColumn);

        // Build the testing panel in the splitpane's right side
        add(buildTestingPanel());


        // Build the test control buttons as small groups (controlsColumn) 
        //   placed within a larger group (controlGroupsColumn) 
        ButtonColumn controlsColumn;
        
        // Base Properties

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("HorizontalRule Properties"));
        
        controlsColumn.addButton("Set Height=null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setHeight(null);
            }
        });
        
        controlsColumn.addButton("Set Height=3px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setHeight(new Extent(3));
            }
        });
        
        
        controlsColumn.addButton("Set Height=10px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setHeight(new Extent(10));
            }
        });

        
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Width"));
        
        controlsColumn.addButton("Set Width=null", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setWidth(null);
            }
        });
        
        controlsColumn.addButton("Set width=300px", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setWidth(new Extent(300));
            }
        });
        
        
        controlsColumn.addButton("Set Height=75%", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setWidth(new Extent(75, Extent.PERCENT));
            }
        });


        // Color
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Background Color"));
       
        controlsColumn.addButton("Random Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                final Color color = StyleUtil.randomColor();
                horizontalRule.setBackground(color);
            }
        });
        
        controlsColumn.addButton("Clear Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setBackground(null);
            }
        });


        // Borders

        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Borders"));
        
        
        controlsColumn.addButton("Random Border", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setBorder(StyleUtil.randomBorder());
            }
        });
        
        controlsColumn.addButton("Random Border Color", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Border border = horizontalRule.getBorder();
                
                if (border == null)
                    border = new Border(new Extent(1), StyleUtil.randomColor(), Border.STYLE_SOLID);
                
                horizontalRule.setBorder(new Border(border.getSize(), StyleUtil.randomColor(), border.getStyle()));
            }
        });
        
        controlsColumn.addButton("Random Border Size", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setBorder(StyleUtil.nextBorderSize(horizontalRule.getBorder()));
            }
        });
        
        controlsColumn.addButton("Random Border Style", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setBorder(StyleUtil.nextBorderStyle(horizontalRule.getBorder()));
            }
        });
        
        controlsColumn.addButton("Clear Border", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setBorder(null);
            }
        });
        
        
        // Enabled/Disabled look
 
        controlsColumn = new ButtonColumn();
        controlGroupsColumn.add(controlsColumn);
        controlsColumn.add(new Label("Enabled/Disabled"));

        controlsColumn.addButton("Toggle Enabled", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean enabled = !horizontalRule.isEnabled();
                horizontalRule.setEnabled(enabled);
            }
        });

        controlsColumn.addButton("Random Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setDisabledBackground(StyleUtil.randomColor());
            }
        });

        controlsColumn.addButton("Clear Disabled Background", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setDisabledBackground(null);
            }
        });
        
        controlsColumn.addButton("Random Disabled Border", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setDisabledBorder(StyleUtil.randomBorder());
            }
        });

        controlsColumn.addButton("Clear Disabled Border", new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                horizontalRule.setDisabledBorder(null);
            }
        });

    }

    private Component buildTestingPanel() {
    
        Column testColumn = new Column();
        testColumn.setStyleName("TestColumn");
        testColumn.add(horizontalRule);
        
        return testColumn;
    }

}
